severity = harsh

[TestingAndDebugging::RequireUseWarnings]
equivalent_modules = common::sense

[TestingAndDebugging::RequireUseStrict]
equivalent_modules = common::sense

[Modules::ProhibitModuleShebang]
severity = gentle
